@@ -16,25 +16,25 @@ class Ingress:
     """
     Contains functions for uploading data to the Osiris-ingress API.
     """
-    # pylint: disable=too-many-arguments
-    def __init__(self, ingress_url: str, tenant_id: str, client_id: str, client_secret: str, dataset_guid: str):
+    def __init__(self,
+                 client_auth: ClientAuthorization,
+                 ingress_url: str,
+                 dataset_guid: str):
         """
+        :param client_auth: The Client Authorization to access the dataset.
         :param ingress_url: The URL to the Osiris-ingress API.
-        :param tenant_id: The tenant ID representing the organisation.
-        :param client_id: The client ID (a string representing a GUID).
-        :param client_secret: The client secret string.
-        :param dataset_guid: The GUID for the dataset.
+        :param dataset_guid: The GUID for the dataset if needed.
         """
-        if None in [ingress_url, tenant_id, client_id, client_secret, dataset_guid]:
+
+        if None in [client_auth, ingress_url, dataset_guid]:
             message = 'One or more of the arguments are None.'
             logger.error(message)
             raise TypeError(message)
 
+        self.client_auth = client_auth
         self.ingress_url = ingress_url
         self.dataset_guid = dataset_guid
 
-        self.client_auth = ClientAuthorization(tenant_id, client_id, client_secret)
-
     def upload_json_file(self, file, schema_validate: bool):
         """
         Uploads the given JSON file to <dataset_guid>.
@@ -52,6 +52,26 @@ class Ingress:
 
         check_status_code(response)
 
+    def upload_json_file_event_time(self, file, event_time: str, schema_validate: bool):
+        """
+        Uploads the given JSON file to <dataset_guid> with a path corresponding to the given event
+        time.
+
+        :param file: The JSON file to upload.
+        :param event_time: Given event time. The path corresponds to this time.
+        :param schema_validate: Validate the content of the file? This requires that the validation schema is
+                                supplied to the DataPlatform.
+        """
+        response = requests.post(
+            url=f'{self.ingress_url}/{self.dataset_guid}/event_time/json',
+            files={'file': file},
+            params=[('schema_validate', schema_validate),
+                    ('event_time', event_time)],
+            headers={'Authorization': self.client_auth.get_access_token()}
+        )
+
+        check_status_code(response)
+
     def upload_file(self, file):
         """
         Uploads the given arbitrary file to <dataset_guid>.
@@ -66,6 +86,24 @@ class Ingress:
 
         check_status_code(response)
 
+    def upload_file_event_time(self, file, event_time: str):
+        """
+        Uploads the given arbitrary file to <dataset_guid> with a path corresponding to the given event
+        time.
+
+        :param file: The arbitrary file to upload.
+        :param event_time: This string must be in the form '[year]-[month]-[day]T[hour]:[minutes] and
+        decides the path where the data is stored.
+        """
+        response = requests.post(
+            url=f'{self.ingress_url}/{self.dataset_guid}/event_time',
+            files={'file': file},
+            params={'event_time': event_time},
+            headers={'Authorization': self.client_auth.get_access_token()}
+        )
+
+        check_status_code(response)
+
     def save_state(self, file):
         """
         Uploads the state file to <dataset_guid>. Can be downloaded from end-point in egress.
