@@ -9,60 +9,49 @@ import pandas as pd
 
 from apache_beam.io import OffsetRangeTracker, iobase
 from azure.core.exceptions import ResourceNotFoundError
-from azure.storage.filedatalake import DataLakeFileClient, FileSystemClient, PathProperties
+from azure.storage.filedatalake import FileSystemClient, PathProperties
 
-from .common import initialize_client_auth
+from .azure_data_storage import Dataset
 from ..core.azure_client_authorization import ClientAuthorization
 
 
-# pylint: disable=too-many-instance-attributes
-class DatalakeFileSource(iobase.BoundedSource):  # noqa
+class FileBatchController:
     """
-    A Class to download files from Azure Datalake
-
-    This datasource can be used in two different ways:
+    Used to find the next batch of files to process.
 
+    This batch controller can be used in two different ways:
     1. You can initialize it with an ingest time. In this case it will process files from the current hour and the
     previous hour no matter if they have been processed before or not. It will also ignore the max_files argument.
-
     2. If you don't initialize it with an ingest time, it will use a state which it will store in a file (STATE_FILE)
     on the dataset. This state will contain the last modified timestamp of the last file it processed. Next time
     it runs it will only process files newer than this timestamp and according to the max_files argument.
         - If you use this approach you need to call close() after the pipeline to save the state otherwise it will
         keep processing the same files again and again.
-        - First time it runs it will use the current UTC time.
-    """
+        - First time it runs it will start from 1970 and parse forward.
+   """
 
     STATE_FILE = 'transformation_state.json'
+    DATE_FORMAT = '%Y-%m-%dT%H:%M:%S'
 
     # pylint: disable=too-many-arguments
     def __init__(self,
-                 tenant_id: str,
-                 client_id: str,
-                 client_secret: str,
+                 client_auth: ClientAuthorization,
                  account_url: str,
                  filesystem_name: str,
                  guid: str,
                  ingest_time: datetime = None,
                  max_files: int = 10):
 
-        if None in [tenant_id, client_id, client_secret, account_url, filesystem_name, guid]:
+        if None in [client_auth, account_url, filesystem_name, guid]:
             raise TypeError
 
-        self.tenant_id = tenant_id
-        self.client_id = client_id
-        self.client_secret = client_secret
+        self.client_auth = client_auth
         self.account_url = account_url
         self.filesystem_name = filesystem_name
         self.guid = guid
         self.has_ingest_time = ingest_time is not None
-
-        # We need to initialize the ClientAuthorization using lazy initializing because of Beam. Beam pickles
-        # this object and pickles only allow simples values as instance variables and not objects.
-        self.client_auth = None
-
-        local_client_auth = ClientAuthorization(tenant_id, client_id, client_secret)
-        self.file_paths = self.__get_file_paths(ingest_time, max_files, local_client_auth)
+        self.last_backfill_start = datetime.utcnow()
+        self.file_paths = self.__get_file_paths(ingest_time, max_files, self.client_auth.get_local_copy())
 
     def __get_file_paths(self, ingest_time: Optional[datetime], max_files: int,
                          local_client_auth: ClientAuthorization) -> List[PathProperties]:
@@ -91,58 +80,136 @@ class DatalakeFileSource(iobase.BoundedSource):  # noqa
     def __get_paths_since_last_run(self, filesystem_client: FileSystemClient, max_files: int) -> List[PathProperties]:
         state = self.__retrieve_transformation_state(filesystem_client)
 
-        if not state:
-            now = datetime.utcnow()
-            folder_path = f'{self.guid}/year={now.year}/month={now.month:02d}/day={now.day:02d}/hour={now.hour:02d}'
+        last_successful_run = datetime.strptime(state['last_successful_run'], self.DATE_FORMAT)
+        if 'last_backfill_start' in state:
+            self.last_backfill_start = datetime.strptime(state['last_backfill_start'], self.DATE_FORMAT)
 
-            paths = self.__get_file_paths_from_folder(folder_path, filesystem_client)
-
-            return paths[:max_files]
-
-        # There is a state file:
-
-        last_successful_run = datetime.strptime(state['last_successful_run'], '%Y-%m-%dT%H:%M:%S')
         now = datetime.utcnow()
         time_range = pd.date_range(last_successful_run, now, freq='H')
-
         paths = []
+
         for timeslot in time_range:
             folder_path = f'{self.guid}/year={timeslot.year}/month={timeslot.month:02d}/day={timeslot.day:02d}' + \
                           f'/hour={timeslot.hour:02d}'
 
-            temp_paths = self.__get_file_paths_from_folder(folder_path, filesystem_client)
-
-            for path in temp_paths:
-                if path.last_modified > last_successful_run:
-                    paths.append(path)
+            paths.extend(self.__get_file_paths_from_folder(folder_path, filesystem_client))
 
             if len(paths) > max_files:
                 break
 
-        return paths[:max_files]
+        paths_return = paths[:max_files]
 
-    def __retrieve_transformation_state(self, filesystem_client: FileSystemClient) -> Optional[Dict]:
+        for path in paths_return:
+            # Set 'processing' tag when the file is getting processed
+            metadata = {'processing': datetime.utcnow().strftime(self.DATE_FORMAT)}
+            filesystem_client.get_file_client(path.name).set_metadata(metadata)
+
+        return paths_return
+
+    def __retrieve_transformation_state(self, filesystem_client: FileSystemClient) -> Dict:
         with filesystem_client.get_file_client(f'{self.guid}/{self.STATE_FILE}') as file_client:
             try:
                 state = file_client.download_file().readall()
                 return json.loads(state)
             except ResourceNotFoundError:
-                return None
+                return {'last_successful_run': '2018-01-01T00:00:00'}
 
     def __save_transformation_state(self, filesystem_client: FileSystemClient, state: Dict):
         with filesystem_client.get_file_client(f'{self.guid}/{self.STATE_FILE}') as file_client:
             json_data = json.dumps(state)
             file_client.upload_data(json_data, overwrite=True)
 
-    @staticmethod
-    def __get_file_paths_from_folder(folder_path: str, file_system_client: FileSystemClient) -> List[PathProperties]:
+    def __get_file_paths_from_folder(self,
+                                     folder_path: str,
+                                     file_system_client: FileSystemClient) -> List[PathProperties]:
         try:
             paths = list(file_system_client.get_paths(path=folder_path))
-            paths.sort(key=lambda x: x.last_modified)
-            return paths
+            unprocessed_files = []
+
+            for path in paths:
+                processed_file_metadata = file_system_client.get_file_client(path.name).get_file_properties().metadata
+                # Only append files which does not have the status=processed or processing tag
+                # NOTE: some "processing" files might not have run if transformation fails. Remove the tag, then rerun
+
+                if 'processed' not in processed_file_metadata:
+                    unprocessed_files.append(path)
+                elif self.last_backfill_start > datetime.strptime(processed_file_metadata['processed'],
+                                                                  self.DATE_FORMAT):
+                    unprocessed_files.append(path)
+
+                # if 'status' not in processed_file_metadata:
+                #     unprocessed_files.append(path)
+                # elif processed_file_metadata['status'] not in ['processed', 'processing']:
+                #     unprocessed_files.append(path)
+
+            return unprocessed_files
         except ResourceNotFoundError:
             return []
 
+    def get_batch(self) -> List[str]:
+        """
+        Get the next batch to process.
+        """
+        # The list of self.file_paths is of type FileProperties and we want to return a list of paths as strings
+        file_path_list = ['/'.join(file_path.name.split('/')[1:]) for file_path in self.file_paths]
+
+        return file_path_list
+
+    def save_state(self):
+        """
+        Updates the transformation state file after a successful run. Its important this method gets called
+        after the pipeline has run or else the datasource will keep processing already processed files.
+        """
+
+        # If ingest time its set, we will not update or use the state. Close() should not be called in this
+        # situation.
+        if self.has_ingest_time:
+            return
+
+        local_client_auth = self.client_auth.get_local_copy()
+
+        with FileSystemClient(self.account_url, self.filesystem_name,
+                              credential=local_client_auth.get_credential_sync()) as filesystem_client:
+            state = self.__retrieve_transformation_state(filesystem_client)
+
+            # state file doesn't exist. We create a fresh one.
+            if not state:
+                state = {}
+
+            if len(self.file_paths):
+                for path in self.file_paths:
+                    # Set 'processed' tag in the metadata of the file
+                    metadata = {'processed': datetime.utcnow().strftime(self.DATE_FORMAT)}
+                    filesystem_client.get_file_client(path.name).set_metadata(metadata)
+
+                # Get the date from the folder structure of the last file it has processed
+                date_elements = self.file_paths[-1].name.split('/')[1:-1]
+                date_str = ''.join([x.split('=')[1] for x in date_elements])
+
+                latest_folder_date = datetime.strptime(date_str, '%Y%m%d%H').isoformat()
+                state['last_successful_run'] = latest_folder_date
+                state['last_backfill_start'] = self.last_backfill_start
+                self.__save_transformation_state(filesystem_client, state)
+
+
+class DatalakeFileSource(iobase.BoundedSource):  # noqa
+    """
+    A Class to download files from Azure Datalake
+
+    Class takes a list of paths to process in the batch.
+    """
+
+    # pylint: disable=too-many-arguments
+    def __init__(self,
+                 dataset: Dataset,
+                 file_paths: List[str]):
+
+        if None in [dataset, file_paths]:
+            raise TypeError
+
+        self.dataset = dataset
+        self.file_paths = file_paths
+
     def estimate_size(self) -> int:
         """
         Returns the number of files to process
@@ -160,7 +227,6 @@ class DatalakeFileSource(iobase.BoundedSource):  # noqa
 
         return OffsetRangeTracker(start_position, stop_position)
 
-    @initialize_client_auth
     def read(self, range_tracker: OffsetRangeTracker) -> Optional[Generator]:
         """
         Returns the content of the next file
@@ -169,14 +235,10 @@ class DatalakeFileSource(iobase.BoundedSource):  # noqa
             if not range_tracker.try_claim(i):
                 return
 
-            path = self.file_paths[i].name
-            with DataLakeFileClient(self.account_url,
-                                    self.filesystem_name, path,
-                                    credential=self.client_auth.get_credential_sync()) as file_client:  # type: ignore
+            path = self.file_paths[i]
+            content = self.dataset.read_file(path)
 
-                content = file_client.download_file().readall()
-
-                yield content
+            yield content
 
     def split(self,
               desired_bundle_size: int,
@@ -200,41 +262,12 @@ class DatalakeFileSource(iobase.BoundedSource):  # noqa
                 stop_position=bundle_stop)
             bundle_start = bundle_stop
 
-    def close(self):
-        """
-        Updates the transformation state file after a successful run. Its important this method gets called
-        after the pipeline has run or else the datasource will keep processing already processed files.
-        """
-
-        # If ingest time its set, we will not update or use the state. Close() should not be called in this
-        # situation.
-        if self.has_ingest_time:
-            return
-
-        local_client_auth = ClientAuthorization(self.tenant_id, self.client_id, self.client_secret)
-
-        with FileSystemClient(self.account_url, self.filesystem_name,
-                              credential=local_client_auth.get_credential_sync()) as filesystem_client:
-            state = self.__retrieve_transformation_state(filesystem_client)
-
-            # state file doesn't exist. We create a fresh one.
-            if not state:
-                state = {}
-
-            if len(self.file_paths):
-                latest_modified = self.file_paths[-1].last_modified  # file_paths is sorted ascending.
-                state['last_successful_run'] = latest_modified.isoformat()
-                self.__save_transformation_state(filesystem_client, state)
-
-
-
 
 class DatalakeFileSourceWithFileName(DatalakeFileSource):  # noqa
     """
     A Class to download files from Azure Datalake with file name.
     """
 
-    @initialize_client_auth
     def read(self, range_tracker: OffsetRangeTracker) -> Optional[Generator]:
         """
         Returns the content of the next file.
@@ -243,12 +276,9 @@ class DatalakeFileSourceWithFileName(DatalakeFileSource):  # noqa
             if not range_tracker.try_claim(i):
                 return
 
-            path = self.file_paths[i].name
-            with DataLakeFileClient(self.account_url,
-                                    self.filesystem_name, path,
-                                    credential=self.client_auth.get_credential_sync()) as file_client:  # type: ignore
-                content = file_client.download_file().readall()
+            path = self.file_paths[i]
+            content = self.dataset.read_file(path)
 
-                file_name = os.path.basename(path)
+            file_name = os.path.basename(path)
 
-                yield file_name, content
+            yield file_name, content
